from .entropy_sampling import EntropySampling  # noqa: F401
from .least_confidence import LeastConfidence  # noqa: F401
from .margin_sampling import MarginSampling  # noqa: F401
from .random_sampling import RandomSampling  # noqa: F401
from .strategy import Strategy  # noqa: F401
